"""
Common functions for Python / Vue / Eel project.

It contains functions for running eel, overriding eel.expose decorator, converting json to correct python
format or transform data into form for vue tables and plots.

Go on

https://mypythontools.readthedocs.io/#project-starter

for example with working examples.

Image of such an app

.. image:: /_static/project-starter-gui.png
    :width: 620
    :alt: project-starter-gui
    :align: center
"""

from mypythontools.pyvueeel.pyvueeel_internal import (
    expose,
    eel,
    expose_error_callback,
    json_to_py,
    run_gui,
    to_table,
    to_vue_plotly,
)

__all__ = [
    "expose",
    "eel",
    "expose_error_callback",
    "json_to_py",
    "run_gui",
    "to_table",
    "to_vue_plotly",
]



"""Tests for helpers module."""

from __future__ import annotations
from pathlib import Path
import sys
import subprocess

root_path = Path(__file__).parents[1].as_posix()  # pylint: disable=no-member
sys.path.insert(0, root_path)


from mypythontools import helpers
from mypythontools import cicd
from mypythontools.helpers.terminal import get_console_str_with_quotes, PYTHON

from typing_extensions import Literal

cicd.tests.setup_tests()

tests_path = Path("tests").resolve()
test_project_path = tests_path / "tested project"

# pylint: disable=missing-function-docstring,


def test_paths():

    assert helpers.paths.PROJECT_PATHS.root == test_project_path
    assert helpers.paths.PROJECT_PATHS.init == test_project_path / "project_lib" / "__init__.py"
    assert helpers.paths.PROJECT_PATHS.app == test_project_path / "project_lib"
    assert helpers.paths.PROJECT_PATHS.docs == test_project_path / "docs"
    assert helpers.paths.PROJECT_PATHS.readme == test_project_path / "README.md"
    assert helpers.paths.PROJECT_PATHS.tests == test_project_path / "tests"


def test_config_argparse():
    """Doctest not tested in config. If changing test, change docs as well."""

    options = [
        {"name": "bool_arg", "input_value": "True", "expected_value": "True", "type": "bool"},
        {"name": "int_arg", "input_value": "666", "expected_value": "666", "type": "int"},
        {"name": "float_arg", "input_value": "666", "expected_value": "666", "type": "float"},
        {"name": "str_arg", "input_value": "666", "expected_value": "666", "type": "str"},
        {"name": "list_arg", "input_value": "[666]", "expected_value": "666", "type": "list"},
        {"name": "dict_arg", "input_value": "\"{'key': 666}\"", "expected_value": "666", "type": "dict"},
    ]

    argparse_script_path = get_console_str_with_quotes(tests_path / "helpers" / "argparse_config.py")

    for i in options:
        output = subprocess.check_output(
            f"{PYTHON} {argparse_script_path} --{i['name']} {i['input_value']}",
            cwd=root_path,
            text=True,
            shell=True,
        ).strip()

        assert all(member in output for member in [i["name"], i["expected_value"], i["type"]])

    get_help = subprocess.check_output(
        f"{PYTHON} {argparse_script_path} --help", cwd=root_path, text=True, shell=True
    ).strip()

    assert "This should be in CLI help" in get_help and "How it works." in get_help

    is_error = False

    try:
        subprocess.check_output(
            f"{PYTHON} {argparse_script_path} --bool_arg", cwd=root_path, text=True, shell=True
        ).strip()
    except Exception:
        is_error = True

    assert is_error, "Value not parsed with correct type, but error not raised"

    is_error = False

    try:
        subprocess.check_output(
            f"{PYTHON} {argparse_script_path} --nonexisting_arg nonsense",
            cwd=root_path,
            text=True,
            shell=True,
        ).strip()
    except Exception:
        is_error = True
    assert is_error, "Non existing variable passed"


if __name__ == "__main__":

    pass



""" Test module. Auto pytest that can be started in IDE or with::

    python -m pytest

in terminal in tests folder.
"""
from . import test_cicd
from . import test_helpers

__all__ = ["test_cicd", "test_helpers"]



"""Module that will help with virtual environments so it's possible to work with venv from python.
You can create, delete or update dependencies."""

from mypythontools.cicd.venvs.venvs_internal import Venv, is_venv

__all__ = ["Venv", "is_venv"]



"""Module with functionality around Continuous Integration and Continuous Delivery.

Subpackages

- build
- deploy
- project_utils
- tests

In project utils you can find many functions for CI/CD, but also pipelining functions that will call them
in defined order.

Why to use this and not Travis or Circle CI? It's local and it's fast. You can setup it as a task in IDE and
if some phase fails, you know it soon and before pushing to repo.

You can also import mypythontools in your CI/CD and use it there of course.
"""
from mypythontools.cicd import build
from mypythontools.cicd import deploy
from mypythontools.cicd import project_utils
from mypythontools.cicd import tests
from mypythontools.cicd import venvs

__all__ = ["build", "deploy", "project_utils", "tests", "venvs"]



"""Module help with many various problems. Subpackages names are self describing and you can find
documentation in subpackages docstrings.
"""
from mypythontools.helpers import config, misc, paths, plots, property, types

__all__ = ["config", "misc", "paths", "plots", "property", "types"]



"""Push the CI pipeline. Format, create commit from all the changes, push and deploy to PyPi."""

# import os
# import inspect
from pathlib import Path
import sys

# Find paths and add to sys.path to be able to use local version and not installed mypythontools version
root_path_str = Path(__file__).parents[1].as_posix()
# root = Path(os.path.abspath(inspect.getframeinfo(inspect.currentframe()).filename)).parents[1]

if root_path_str not in sys.path:
    sys.path.insert(0, root_path_str)

from mypythontools import cicd

if __name__ == "__main__":
    # All the parameters can be overwritten via CLI args
    # cicd.project_utils.project_utils_pipeline(
    #     reformat=True,
    #     test=True,
    #     test_options={"virtualenvs": ["venv/37", "venv/310"]},
    #     version="increment",
    #     docs=True,
    #     sync_requirements=False,
    #     commit_and_push_git=True,
    #     commit_message="New commit",
    #     tag="__version__",
    #     tag_message="New version",
    #     deploy=True,
    #     allowed_branches=("master", "main"),
    # )

    cicd.project_utils.project_utils_pipeline(do_only="deploy")



"""Tests for cicd module."""

# pylint: disable=missing-function-docstring


from __future__ import annotations
import shutil
from pathlib import Path
import sys
import os

root_path = Path(__file__).parents[1].as_posix()  # pylint: disable=no-member
sys.path.insert(0, root_path)


from mypythontools import cicd
from mypythontools.helpers.paths import PROJECT_PATHS

test_project_path = Path("tests").resolve() / "tested project"


def test_docs_regenerate():
    rst_path = test_project_path / "docs" / "source" / "project_lib.rst"
    another_rst_path = test_project_path / "docs" / "source" / "content" / "also_not_deleted.rst"
    not_deleted = test_project_path / "docs" / "source" / "not_deleted.rst"

    if rst_path.exists():
        rst_path.unlink()  # missing_ok=True from python 3.8 on...

    if not not_deleted.exists():
        with open(not_deleted, "w") as not_deleted_file:
            not_deleted_file.write("I will not be deleted.")
            # missing_ok=True from python 3.8 on...

    cicd.project_utils.project_utils_functions.docs_regenerate(
        keep=("conf.py", "index.rst", "_static", "_templates", "content/**", "not_deleted.rst")
    )

    assert rst_path.exists()
    assert another_rst_path.exists()
    assert not_deleted.exists()


def test_reformat_with_black():
    cicd.project_utils.project_utils_functions.reformat_with_black()


def test_set_version():
    cicd.project_utils.project_utils_functions.set_version("0.0.2")
    assert cicd.project_utils.project_utils_functions.get_version() == "0.0.2"
    cicd.project_utils.project_utils_functions.set_version("0.0.1")


def test_build():

    # Build app with pyinstaller example
    cicd.build.build_app(
        main_file="app.py",
        console=True,
        debug=True,
        clean=False,
        build_web=False,
        ignored_packages=["matplotlib"],
        virtualenv=sys.prefix,
        sync_requirements=None,
    )

    assert (test_project_path / "dist").exists()

    shutil.rmtree(test_project_path / "build")
    shutil.rmtree(test_project_path / "dist")


def test_add_readme_tests():
    for i in PROJECT_PATHS.tests.glob("*"):
        if i.name.startswith("test_readme_generated"):
            i.unlink()

    cicd.tests.add_readme_tests()
    for i in PROJECT_PATHS.tests.glob("*"):
        if i.name.startswith("test_readme_generated"):
            print("Readme tests found.")


def test_project_utils_pipeline():
    cicd.project_utils.project_utils_pipeline(
        commit_and_push_git=False,
        deploy=False,
        allowed_branches=None,
        test_options={"virtualenvs": [sys.prefix], "sync_requirements": None},
    )


if __name__ == "__main__":
    # Find paths and add to sys.path to be able to import local modules
    cicd.tests.setup_tests()

    test_project_path = Path("tests").resolve() / "tested project"
    os.chdir(test_project_path)

    # test_cicd()
    # test_build()

    test_project_utils_pipeline()
